public class Miller {

    public static boolean vonNeumann(double[][] f){
	double f0abs2 = Complex.abs2(eval(f, new double[2]));
	double[][] fStar = starPolynomial(f);
	double fStar0abs2 = Complex.abs2(eval(fStar, new double[2]));

	double[][] fOne = onePolynomial(f, fStar);
	
	double[][] fPrime = primePolynomial(f);
	
	return (
		(isConstant(f)) ||
		((isZero(fOne)) && (vonNeumann(fPrime))) ||
		((fStar0abs2 > f0abs2) && (vonNeumann(fOne)))
		);
    }

    
    protected static double[][] starPolynomial(double[][] poly){
	double[][] res = new double[poly.length][2];
	
	for (int i = 0; i < poly.length; i++){
	    res[i][Complex.RE] =  poly[poly.length-1-i][Complex.RE];
	    res[i][Complex.IM] = -poly[poly.length-1-i][Complex.IM];
	}
	return res;
    }

    protected static double[][] primePolynomial(double[][] poly){
	if (poly.length == 1){
	    return new double[1][2];
	} else {
	    double[][] res = new double[poly.length-1][2];
	    
	    for (int i = 0; i < res.length; i++){
		res[i][Complex.RE] = poly[i+1][Complex.RE] * (i+1);
		res[i][Complex.IM] = poly[i+1][Complex.IM] * (i+1);
	    }
	    return res;
	}
    }
    
    protected static double[][] onePolynomial(double[][] poly, double[][] starPoly){
	if (poly.length == 1){
	    return new double[1][2];
	} else {
	    double[][] res = new double[poly.length-1][2];
	    double[][] tmp1 = multiply(poly, starPoly[0]);
	    double[][] tmp2 = multiply(starPoly, poly[0]);
	    
	    for (int i = 0; i < res.length; i++){
		res[i][Complex.RE] = tmp1[i+1][Complex.RE] - tmp2[i+1][Complex.RE];
		res[i][Complex.IM] = tmp1[i+1][Complex.IM] - tmp2[i+1][Complex.IM];
	    }
	    return res;
	}
    }

    // trivial polynomial methods

    protected static boolean isZero(double[][] poly){
	if (poly == null){ return true; } else {
	    boolean zero = true;
	    for (int i = 0; i < poly.length; i++){
		if ((Math.abs(poly[i][Complex.RE]) > 1e-14) || (Math.abs(poly[i][Complex.IM]) > 1e-14)){ zero = false; break; }
	    }
	    return zero;
	}
    }

    protected static boolean isConstant(double[][] poly){
	return isZero(primePolynomial(poly));
    }

    public static double[] eval(double[][] poly, double[] x){
	double[] val = new double[2];
	double[] powX = new double[2];
	powX[Complex.RE] = 1.0;
	
	for (int i = 0; i < poly.length; i++){
	    val = Complex.getSum(val, Complex.getProduct(poly[i], powX));
	    powX = Complex.getProduct(powX, x);
	}
	return val;
    
    }

    protected static double[][] multiply(double[][] poly, double[] a){
	double[][] res = new double[poly.length][2];
	for (int i = 0; i < poly.length; i++){
	    res[i] = Complex.getProduct(poly[i], a);
	}
	return res;
    
    }
    

}
